#!/usr/bin/env python
# vim: set ts=4 sts=4 sw=4 textwidth=92:

from RarslaveCommon import *

# PAR2Set-derived types
import PAR2Set_JOIN
import PAR2Set_ZIP
import PAR2Set_OLDRAR
import PAR2Set_NEWRAR
import PAR2Set_EF_OLDRAR
import PAR2Set_EF_NEWRAR

import logging

class RarslaveDetector (object):

	# A tuple of tuples with the following definition:
	# (TYPE_NAME, DETECTION_FUNCTION, PAR2Set-derived class)

	TYPES = (	(PAR2Set_JOIN.detect_JOIN, PAR2Set_JOIN.PAR2Set_JOIN),
				(PAR2Set_ZIP.detect_ZIP, PAR2Set_ZIP.PAR2Set_ZIP),
				(PAR2Set_OLDRAR.detect_OLDRAR, PAR2Set_OLDRAR.PAR2Set_OLDRAR),
				(PAR2Set_NEWRAR.detect_NEWRAR, PAR2Set_NEWRAR.PAR2Set_NEWRAR),
				(PAR2Set_EF_OLDRAR.detect_EF_OLDRAR, PAR2Set_EF_OLDRAR.PAR2Set_EF_OLDRAR),
				(PAR2Set_EF_NEWRAR.detect_EF_NEWRAR, PAR2Set_EF_NEWRAR.PAR2Set_EF_NEWRAR),
			)

	def __init__ (self, dir, p2file):

		# The real "meat" of the class
		self.dir = dir
		self.p2file = p2file
		self.basename = get_basename (p2file)

		# Find files that match by name only
		self.name_matched_files = find_name_matches (self.dir, self.basename)

		# Find all par2 files for this set using name matches
		self.all_p2files = find_par2_files (self.name_matched_files)

		# Try to get the protected files for this set
		self.prot_matched_files = parse_all_par2 (self.dir, self.p2file, self.all_p2files)

	def runMatchingTypes (self):
		# Now tries to run every type of PAR2Set-derived class for which the detector
		# detects that the class is valid.

		detected = False

		for (detector, classname) in self.TYPES:
			if detector (self.name_matched_files, self.prot_matched_files):
				# The detector matched, so we're up and running!

				p2set = classname (self.dir, self.p2file)

				detected = True
				logging.debug ('Detected type: %s' % p2set)

				# Try to have rarslave do it's thing
				ret = p2set.runAll ()

				# If something already worked, there is no need to continue,
				# since we've already finished!
				if ret == SUCCESS:
					break
				else:
					logging.error ('Detected type failed for: %s' % self.p2file)

		# Make sure we detected at least one valid type
		if not detected:
			logging.warning ('Unable to determine type: %s' % self.p2file)
			logging.debug ('The following information will help in writing a detector:')
			logging.debug ('name_matches: %s' % self.name_matched_files)
			logging.debug ('prot_matches: %s' % self.prot_matched_files)
			return -EDETECT

		# Make sure that something worked
		if ret != SUCCESS:
			logging.critical ('All types failed for: %s' % self.p2file)

		return ret

def main ():
	pass

if __name__ == '__main__':
	main ()

