#!/usr/bin/env python
# vim: set ts=4 sts=4 sw=4 textwidth=112 :

VERSION="2.0.0"
PROGRAM="rarslave2"

import os, sys, optparse, logging
import RarslaveDetector
import RarslaveGlobals
from RarslaveCommon import *

# Global options from the RarslaveGlobals class
options = RarslaveGlobals.options
config = RarslaveGlobals.config

# A tiny class to hold logging output until we're finished
class DelayedLogger (object):
	def __init__ (self, output=sys.stdout.write):
		self.__messages = []
		self.__output = output

	def write (self, msg):
		self.__messages.append (msg)

	def flush (self):
		pass

	def size (self):
		"""Returns the number of messages queued for printing"""
		return len (self.__messages)

	def close (self):
		"""Print all messages, clear the queue"""
		for m in self.__messages:
			self.__output (m)

		self.__messages = []

# A tiny class used to find unique PAR2 sets
class CompareSet (object):

	def __init__ (self, dir, p2file):
		self.dir = dir
		self.p2file = p2file

		self.basename = get_basename (self.p2file)
		self.name_matches = find_name_matches (self.dir, self.basename)

	def __eq__ (self, rhs):
		return (self.dir == rhs.dir) \
				and (self.basename == rhs.basename) \
				and list_eq (self.name_matches, rhs.name_matches)


def find_all_par2_files (dir):
	"""Finds all par2 files in a directory"""
	# NOTE: does NOT return absolute paths

	if not os.path.isdir (os.path.abspath (dir)):
		raise ValueError # bad directory given

	dir = os.path.abspath (dir)
	files = os.listdir (dir)

	return find_par2_files (files)

def generate_all_parsets (dir):
	# Generate all parsets in the given directory.

	assert os.path.isdir (dir) # Directory MUST be valid

	parsets = []
	p2files = find_all_par2_files (dir)

	for f in p2files:
		p = CompareSet (dir, f)
		if p not in parsets:
			parsets.append (p)

	return [(p.dir, p.p2file) for p in parsets]

def check_required_progs():
	"""Check if the required programs are installed"""

	shell_not_found = 32512
	needed = []

	if run_command ('par2repair --help > /dev/null 2>&1') == shell_not_found:
		needed.append ('par2repair')

	if run_command ('unrar --help > /dev/null 2>&1') == shell_not_found:
		needed.append ('unrar')

	if run_command ('unzip --help > /dev/null 2>&1') == shell_not_found:
		needed.append ('unzip')

	if needed:
		for n in needed:
			print 'Needed program "%s" not found in $PATH' % (n, )

		sys.exit(1)

def run_options (options):

	# Fix directories
	options.work_dir = full_abspath (options.work_dir)

	# Make sure that the directory is valid
	if not os.path.isdir (options.work_dir):
		sys.stderr.write ('\"%s\" is not a valid directory. Use the \"-d\"\n' % options.work_dir)
		sys.stderr.write ('option to override the working directory temporarily, or edit the\n')
		sys.stderr.write ('configuration file to override the working directory permanently.\n')
		sys.exit (1)

	if options.extract_dir != None:
		options.extract_dir = full_abspath (options.extract_dir)

	if options.version:
		print PROGRAM + ' - ' + VERSION
		print
		print 'Copyright (c) 2005,2006 Ira W. Snyder (devel@irasnyder.com)'
		print
		print 'This program comes with ABSOLUTELY NO WARRANTY.'
		print 'This is free software, and you are welcome to redistribute it'
		print 'under certain conditions. See the file COPYING for details.'
		sys.exit (0)

	if options.check_progs:
		check_required_progs ()

	if options.write_def_config:
		config.write_config (default=True)

	if options.write_config:
		config.write_config ()

def find_loglevel (options):

	loglevel = options.verbose - options.quiet

	if loglevel > 1:
		loglevel = 1

	if loglevel < -3:
		loglevel = -3

	LEVELS = {	1 : logging.DEBUG,
				0 : logging.INFO,
				-1: logging.WARNING,
				-2: logging.ERROR,
				-3: logging.CRITICAL
	}

	return LEVELS [loglevel]

def main ():

	# Setup the logger
	logger = DelayedLogger ()
	logging.basicConfig (stream=logger, level=logging.WARNING, \
			format='%(levelname)-8s %(message)s')

	# Build the OptionParser
	parser = optparse.OptionParser()
	parser.add_option('-n', '--not-recursive',
						action='store_false', dest='recursive',
						default=config_get_value('options', 'recursive'),
						help="Don't run recursively")

	parser.add_option('-d', '--work-dir',
						dest='work_dir', type='string',
						default=config_get_value('directories', 'working_directory'),
						help="Start running at DIR", metavar='DIR')

	parser.add_option('-e', '--extract-dir',
						dest='extract_dir', type='string',
						default=config_get_value('directories', 'extract_directory'),
						help="Extract to DIR", metavar='DIR')

	parser.add_option('-p', '--check-required-programs',
						action='store_true', dest='check_progs',
						default=False,
						help="Check for required programs")

	parser.add_option('-f', '--write-default-config',
						action='store_true', dest='write_def_config',
						default=False, help="Write out a new default config")

	parser.add_option('-c', '--write-new-config',
						action='store_true', dest='write_config',
						default=False, help="Write out the current config")

	parser.add_option('-i', '--interactive', dest='interactive', action='store_true',
						default=config_get_value('options', 'interactive'),
						help="Confirm before removing files")

	parser.add_option('-q', '--quiet', dest='quiet', action='count',
						default=0, help="Output fatal messages only")

	parser.add_option('-v', '--verbose', dest='verbose', action='count',
						default=0, help="Output extra information")

	parser.add_option('-V', '--version', dest='version', action='store_true',
						default=False, help="Output version information")

	parser.version = VERSION

	# Parse the given options
	global options
	(RarslaveGlobals.options, args) = parser.parse_args()
	options = RarslaveGlobals.options

	# Run any special actions that are needed on these options
	run_options (options)

	# Find the loglevel using the options given 
	logging.getLogger().setLevel (find_loglevel (options))

	# Run recursively
	if options.recursive:
		for (dir, subdirs, files) in os.walk (options.work_dir):
			parsets = generate_all_parsets (dir)
			for (p2dir, p2file) in parsets:
				detector = RarslaveDetector.RarslaveDetector (p2dir, p2file)
				ret = detector.runMatchingTypes ()

	# Non-recursive
	else:
		parsets = generate_all_parsets (options.work_dir)
		for (p2dir, p2file) in parsets:
			detector = RarslaveDetector.RarslaveDetector (p2dir, p2file)
			ret = detector.runMatchingTypes ()

	# Print the results
	if logger.size () > 0:
		print 'Log\n' + '=' * 80
		logger.close ()

	# Done!
	return 0

if __name__ == '__main__':
	main ()

